##############################################################################
#
# Copyright (c) 2003-2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""fields

$Id: field.py 4634 2006-01-06 20:21:15Z fred $
"""
from zope import interface, i18n, schema, component
from zope.schema.interfaces import (
    ValidationError, WrongType, IField, IVocabularyTokenized)
from zope.interface.exceptions import DoesNotImplement
import zope.app.catalog.interfaces
import zope.index.text.queryparser
import zope.index.text.parsetree

from zc.form import interfaces
from zc.form.i18n import _

_no_unioned_field_validates = _(
    "No unioned field validates ${value}.")

_range_less_error = _("${minimum} must be less than ${maximum}.")
_range_less_equal_error = _(
    "${minimum} must be less than or equal to ${maximum}.")
_combination_wrong_size_error = _("The value has the wrong number of members")
_combination_not_a_sequence_error = _("The value is not a sequence")
_bad_query = _("Invalid query.")

# Union field that accepts other fields...

class MessageValidationError(ValidationError):
    "ValidationError that takes a message"

    def __init__(self, message, mapping=None):
        if mapping is not None:
            self.message = i18n.Message(message, mapping=mapping)
        else:
            self.message = message
        self.args = (message, mapping)
    def doc(self):
        return self.message

class BaseField(schema.Field):
    """Field with a callable as default and a tuple of constraints.

    >>> def secure_password(field, value):
    ...     if len(value) < 8:
    ...         raise schema.ValidationError, 'Password too short.'
    ...
    >>> class IDummy(interface.Interface):
    ...     suggested_password = BaseField(
    ...         title=u'Suggested Password',
    ...         default_getter=lambda context: u'asdf',
    ...         constraints=(secure_password, ))
    ... 
    >>> f = IDummy['suggested_password'].bind(None) # use None as context
    >>> interfaces.IExtendedField.providedBy(f)
    True
    >>> f.__name__
    'suggested_password'
    >>> f.title
    u'Suggested Password'
    >>> f.default
    u'asdf'
    >>> f.validate(u'123456789')
    >>> f.validate(u'asdf')
    Traceback (most recent call last):
    ...
    ValidationError: Password too short.
    """

    interface.implements(interfaces.IExtendedField)

    constraints = ()
    _default = default_getter = None
    
    def __init__(self, constraints=(), default_getter=None, **kw):
        self.constraints = constraints
        if default_getter is not None and 'default' in kw:
            raise TypeError(
                'may not specify both a default and a default_getter')
        super(BaseField, self).__init__(**kw)
        self.default_getter = default_getter

    def _validate(self, value):
        super(BaseField, self)._validate(value)
        if value != self.missing_value:
            for constraint in self.constraints:
                constraint(self, value)

    @apply
    def default():
        def getter(self):
            if self.default_getter is not None:
                return self.default_getter(self.context)
            else:
                return self._default
        def setter(self, value):
            assert self.default_getter is None
            self._default = value
        return property(getter, setter)

class Option(BaseField):

    interface.implements(interfaces.IOptionField)

    def __init__(self, value=None, value_getter=None, 
                 identity_comparison=False, **kw):
        self.value = value
        self.value_getter = value_getter
        self.identity_comparison = identity_comparison
        assert (value is None) ^ (value_getter is None)
        assert not kw.get('required')
        kw['required'] = False
        super(Option, self).__init__(**kw)

    def _validate(self, value):
        if value != self.missing_value:
            if self.identity_comparison:
                if self.getValue() is not value:
                    raise WrongType
            elif self.getValue() != value:
                raise WrongType

    def getValue(self):
        if self.value_getter is not None:
            return self.value_getter(self.context)
        else:
            return self.value

class Union(BaseField):
    """
    The Union field allows a schema field to hold one of many other field 
    types.  For instance, you might want to make a field that can hold
    a duration *or* a date, if you are working with a PIM app.  Or perhaps 
    you want to have a field that can hold a string from a vocabulary *or* a
    custom string.  Both of these examples can be accomplished in a variety of
    ways--the union field is one option.
    
    The second example is more easily illustrated.  Here is a union field that
    is a simple way of allowing "write-ins" in addition to selections from a 
    choice.  We'll be very explicit about imports, in part so this can be
    trivially moved to a doc file test.
    
    Notice as you look through the example that field order does matter: the
    first field as entered in the field list that validates is chosen as the
    validField; thus, even though the options in the Choice field would also 
    validate in a TextLine, the Choice field is identified as the "validField"
    because it is first.
    
    >>> class IDummy(interface.Interface):
    ...     cartoon_character = Union((
    ...         schema.Choice(
    ...             title=u'Disney',
    ...             description=u'Some tried-and-true Disney characters',
    ...             values=(u'Goofy',u'Mickey',u'Donald',u'Minnie')),
    ...         schema.TextLine(
    ...             title=u'Write-in',
    ...             description=u'Name your own!')),
    ...         required=True,
    ...         title=u'Cartoon Character',
    ...         description=u'Your favorite cartoon character')
    ... 
    >>> f = IDummy['cartoon_character']
    >>> interfaces.IUnionField.providedBy(f)
    True
    >>> f.__name__
    'cartoon_character'
    >>> isinstance(f.fields[0], schema.Choice)
    True
    >>> isinstance(f.fields[1], schema.TextLine)
    True
    >>> f.fields[0].__name__ != f.fields[1].__name__
    True
    >>> len(f.fields)
    2
    >>> f.title
    u'Cartoon Character'
    >>> f.validate(u'Goofy')
    >>> f.validField(u'Goofy') is f.fields[0]
    True
    >>> f.validate(u'Calvin')
    >>> f.validField(u'Calvin') is f.fields[1]
    True
    >>> f.validate(42)
    Traceback (most recent call last):
    ...
    MessageValidationError: (u'No unioned field validates ${value}.', {'value': 42})

    That's a working example.  Now lets close with a couple of examples that
    should fall over.

    You must union at least two fields:

    >>> f = Union((schema.TextLine(title=u'Foo Text Line!'),), title=u'Foo')
    Traceback (most recent call last):
    ...
    ValueError: union must combine two or more fields

    And, unsurprisingly, they must actually be fields:
    
    >>> from zope.interface.exceptions import DoesNotImplement
    >>> try:
    ...     f = Union(('I am not a number.', 'I am a free man!'), title=u'Bar')
    ... except DoesNotImplement:
    ...     print "Not a field"
    ... 
    Not a field

    Binding a union field also takes care of binding the contained fields:

    >>> context = object()
    >>> bound_f = f.bind(context)
    >>> bound_f.context is context
    True
    >>> bound_f.fields[0].context is context
    True
    >>> bound_f.fields[1].context is context
    True
    """
    
    interface.implements(interfaces.IUnionField)
    
    fields = ()
    use_default_for_not_selected = False
    
    def __init__(self, fields, use_default_for_not_selected=False, **kw):
        if len(fields) < 2:
            raise ValueError(_("union must combine two or more fields"))
        for ix, field in enumerate(fields):
            if not IField.providedBy(field):
                raise DoesNotImplement(IField)
            field.__name__ = "unioned_%02d" % ix
        self.fields = tuple(fields)
        self.use_default_for_not_selected = use_default_for_not_selected
        super(Union, self).__init__(**kw)

    def bind(self, object):
        clone = super(Union, self).bind(object)
        # We need to bind the fields too
        clone_fields = []
        for field in clone.fields:
            clone_fields.append(field.bind(object))
        clone_fields = tuple(clone_fields)
        clone.fields = clone_fields
        return clone

    def validField(self, value):
        "returns first valid field, or None"
        for field in self.fields:
            try:
                field.validate(value)
            except ValidationError:
                pass
            else:
                return field
    
    def _validate(self, value):
        if self.validField(value) is None:
            raise MessageValidationError(_no_unioned_field_validates, 
                                         {'value': value})

class OrderedCombinationConstraint(object):
    def __init__(self, may_be_equal=True, decreasing=False):
        self.may_be_equal = may_be_equal
        self.decreasing = decreasing

    def __call__(self, field, value):
        # can assume that len(value) == len(field.fields)
        last = None
        for v, f in map(None, value, field.fields):
            if v != f.missing_value:
                if last is not None:
                    if self.decreasing:
                        if self.may_be_equal:
                            if v > last:
                                raise MessageValidationError(
                                    _range_less_equal_error,
                                    {'minimum': v, 'maximum': last})
                        elif v >= last:
                            raise MessageValidationError(
                                _range_less_error,
                                {'minimum': v, 'maximum': last})
                    else:
                        if self.may_be_equal:
                            if v < last:
                                raise MessageValidationError(
                                    _range_less_equal_error,
                                    {'minimum': last, 'maximum': v})
                        elif v <= last:
                            raise MessageValidationError(
                                _range_less_error,
                                {'minimum': last, 'maximum': v})
                last = v
        

class Combination(BaseField):
    """a combination of two or more fields, all of which may be completed.
    
    It accepts two or more fields.  It also accepts a 'constraints' argument.
    Unlike the usual 'constraint' argument (which is also available), the
    constraints should be a sequence of callables that take a field and a
    value, and they should raise an error if there is a problem.
    
    >>> from zc.form.field import Combination, OrderedCombinationConstraint
    >>> from zope import schema, interface
    >>> class IDemo(interface.Interface):
    ...     publication_range = Combination(
    ...         (schema.Date(title=u'Begin', required=False),
    ...          schema.Date(title=u'Expire', required=True)),
    ...         title=u'Publication Range',
    ...         required=True,
    ...         constraints=(OrderedCombinationConstraint(),))
    ...
    >>> f = IDemo['publication_range']
    >>> interfaces.ICombinationField.providedBy(f)
    True
    >>> f.__name__
    'publication_range'
    >>> isinstance(f.fields[0], schema.Date)
    True
    >>> isinstance(f.fields[1], schema.Date)
    True
    >>> f.title
    u'Publication Range'
    >>> f.fields[0].title
    u'Begin'
    >>> f.fields[1].title
    u'Expire'
    >>> import datetime
    >>> f.validate((datetime.date(2005, 6, 22), datetime.date(2005, 7, 10)))
    >>> f.validate((None, datetime.date(2005, 7, 10)))
    >>> f.validate((datetime.date(2005, 6, 22), None))
    Traceback (most recent call last):
    ...
    RequiredMissing
    >>> f.validate(('foo', datetime.date(2005, 6, 22)))
    Traceback (most recent call last):
    ...
    WrongType: ('foo', <type 'datetime.date'>)
    >>> f.validate('foo') # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    MessageValidationError: (u'The value has the wrong number of members', ...)
    >>> f.validate(17)
    Traceback (most recent call last):
    ...
    MessageValidationError: (u'The value is not a sequence', None)
    >>> f.validate((datetime.date(2005, 6, 22), datetime.date(1995, 7, 10)))
    ... # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    MessageValidationError: (u'${minimum} must be less than or equal to ...
    """

    interface.implements(interfaces.ICombinationField)

    fields = constraints = ()
    
    def __init__(self, fields, **kw):
        if len(fields) < 2:
            raise ValueError(_("combination must combine two or more fields"))
        for ix, field in enumerate(fields):
            if not IField.providedBy(field):
                raise DoesNotImplement(IField)
            field.__name__ = "combination_%02d" % ix
        self.fields = tuple(fields)
        super(Combination, self).__init__(**kw)

    def _validate(self, value):
        if value != self.missing_value:
            try:
                len_value = len(value)
            except (TypeError, AttributeError):
                raise MessageValidationError(
                    _combination_not_a_sequence_error)
            if len_value != len(self.fields):
                raise MessageValidationError(
                    _combination_wrong_size_error)
            for v, f in map(None, value, self.fields):
                f = f.bind(self.context)
                f.validate(v)
        super(Combination, self)._validate(value)

class QueryTextLineConstraint(BaseField, schema.TextLine):
    def __init__(self, index_getter=None, catalog_name=None, index_name=None):
        assert not ((catalog_name is None) ^ (index_name is None))
        assert (index_getter is None) ^ (catalog_name is None)
        self.catalog_name = catalog_name
        self.index_name = index_name
        self.index_getter = index_getter

    def __call__(self, field, value):
        if self.index_getter is not None:
            index = self.index_getter(self.context)
        else:
            catalog = component.getUtility(
                zope.app.catalog.interfaces.ICatalog,
                self.catalog_name,
                field.context)
            index = catalog[self.index_name]
        parser = zope.index.text.queryparser.QueryParser(index.lexicon)
        try:
            tree = parser.parseQuery(value)
        except zope.index.text.parsetree.ParseError, e:
            raise MessageValidationError(_bad_query)

class TextLine(BaseField, schema.TextLine):
    """An extended TextLine.

    >>> from zope.index.text.textindex import TextIndex
    >>> index = TextIndex()
    >>> class IDemo(interface.Interface):
    ...     query = TextLine(
    ...         constraints=(
    ...             QueryTextLineConstraint(
    ...                 lambda context: index),),
    ...         title=u"Text Query")
    ...
    >>> field = IDemo['query'].bind(None) # using None as context
    >>> field.validate(u'cow')
    >>> field.validate(u'cow and dog')
    >>> field.validate(u'a the') # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    MessageValidationError: (u'Invalid query.', None)
    >>> field.validate(u'and') # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    MessageValidationError: (u'Invalid query.', None)
    >>> field.validate(u'cow not dog')
    >>> field.validate(u'cow not not dog') # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    MessageValidationError: (u'Invalid query.', None)
    >>> field.validate(u'cow and not dog')
    >>> field.validate(u'cow -dog')
    >>> field.validate('cow') # non-unicode fails, as usual with TextLine
    Traceback (most recent call last):
    ...
    WrongType: ('cow', <type 'unicode'>)
    """

class HTMLSnippet(BaseField, schema.Text):
    interface.implements(interfaces.IHTMLSnippet)

class HTMLDocument(BaseField, schema.Text):
    interface.implements(interfaces.IHTMLDocument)



import pytz
import zc.sourcefactory.basic

class USTimeZones(
    zc.sourcefactory.basic.BasicSourceFactory):

    """List of timezones taken from pytz"""

    tzs = [tz for tz in pytz.common_timezones if 'US/' in tz and
                                            'Pacific-New' not in tz]

    def getValues(self):
        return self.tzs

    def getTitle(self, value):
        return value

    def getToken(self, value):
        return value




##############################################################################
#
# Copyright (c) 2003-2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Union widget

$Id: unionwidget.py 4634 2006-01-06 20:21:15Z fred $
"""

from zope import component
from zope.app.form.interfaces import WidgetInputError

from widgetapi import BaseWidget
from zope.app.form.interfaces import IInputWidget
from zope.app.pagetemplate import ViewPageTemplateFile

from zope.formlib import namedtemplate

import zc.form.interfaces

from zc.form.i18n import _


class CompositeOptionWidget(BaseWidget):
    def __call__(self):
        return None

class NotChosenWidget(object):
    error = name = None
    required = False

    def __init__(self, label, hint):
        self.label = label
        self.hint = hint

    def __call__(self):
        return None

class UnionWidget(BaseWidget):
    
    _field_index = None
    
    no_value_label = _('union_field_label-no_value', "Not specified")
    no_value_hint = _('union_field_hint-no_value', '')
    
    def loadValueFromRequest(self):
        field = self.context
        missing_value = field.missing_value
        value = self.request.form.get(self.name)
        try:
            value = int(value)
        except (TypeError, ValueError):
            value = missing_value
        else:
            if value >= len(field.fields):
                value = missing_value
            else:
                self._field_index = value
                # value should be an int index of the active field
                active = field.fields[value].bind(self.context)
                if zc.form.interfaces.IOptionField.providedBy(active):
                    return active.getValue()
                widget = component.getMultiAdapter(
                    (active, self.request), IInputWidget)
                widget.required = widget.context.required = self.required
                widget.setPrefix(self.name)
                try:
                    return widget.getInputValue()
                except WidgetInputError, e:
                    # recast with our name and title
                    self._error = WidgetInputError(
                        self.context.__name__,
                        self.label,
                        e.errors)
        return missing_value
    
    template = namedtemplate.NamedTemplate('default')
    
    def render(self, value):
        # choices = ({selected, identifier, widget},)
        # widget may be None, name may be None.
        field = self.context
        missing_value = field.missing_value
        choices = []
        field_index = self._field_index
        if field_index is not None:
            chosen_field = field.fields[self._field_index]
        elif value is not missing_value:
            chosen_field = field.validField(value)
        else:
            chosen_field = None
        for ix, inner_field in enumerate(field.fields):
            selected = inner_field is chosen_field
            inner = inner_field.bind(field.context)
            identifier = "%s-%02d" % (self.name, ix)
            if zc.form.interfaces.IOptionField.providedBy(inner):
                widget = CompositeOptionWidget(inner, self.request)
            else:
                widget = component.getMultiAdapter(
                    (inner, self.request), IInputWidget)
                if selected:
                    widget.setRenderedValue(value)
                elif self._renderedValueSet():
                    if field.use_default_for_not_selected:
                        widget.setRenderedValue(inner.default)
                    else:
                        widget.setRenderedValue(inner.missing_value)
            widget.setPrefix(self.name)
            choices.append(
                {'selected': selected, 'identifier': identifier,
                 'widget': widget, 'value': unicode(ix)})
        if not field.required:
            ix += 1
            selected = chosen_field is None
            identifier = "%s-%02d" % (self.name, ix)
            widget = NotChosenWidget(self.no_value_label, self.no_value_hint)
            choices.append(
                {'selected': selected, 'identifier': identifier,
                 'widget': widget, 'value': unicode(ix)})
        return self.template(choices=choices)

default_template = namedtemplate.NamedTemplateImplementation(
    ViewPageTemplateFile('unionwidget.pt'), UnionWidget)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""source input widget with most recently used (MRU) value support

$Id: mruwidget.py 3677 2005-10-19 07:10:22Z fred $
"""
import cgi

import persistent.list
from BTrees import OOBTree

import zope.app.form.interfaces
import zope.app.form.browser.interfaces
from zope.app import zapi 
from zope.app.form.browser.source import SourceInputWidget
from zope.schema.interfaces import ISourceQueriables, ValidationError
import zope.annotation.interfaces

import zc.resourcelibrary

class MruSourceInputWidget(SourceInputWidget):
    ANNOTATION_KEY = 'zc.form.browser.mruwidget'

    def hasInput(self):
        return self.name+'.displayed' in self.request

    def getMostRecentlyUsedTokens(self):
        """Get a sequence of the most recently used tokens (most recent first).
        """
        key = self.name # TODO should the key be more specific?
        principal = self.request.principal
        annotations = zope.annotation.interfaces.IAnnotations(principal)

        annotation = annotations.get(self.ANNOTATION_KEY)
        if annotation is None:
            annotations[self.ANNOTATION_KEY] = annotation = OOBTree.OOBTree()

        tokens = annotation.get(key)
        if tokens is None:
            tokens = annotation[key] = persistent.list.PersistentList()

        return tokens
        
    def getMostRecentlyUsedTerms(self):
        """Get a sequence of the most recently used terms (most recent first).
        """
        tokens = self.getMostRecentlyUsedTokens()
        terms = zapi.getMultiAdapter((self.source, self.request),
                                     zope.app.form.browser.interfaces.ITerms)
        mru = []
        for token in tokens:
            try:
                value = terms.getValue(token)
                term = terms.getTerm(value)
            except LookupError:
                continue

            mru.append(term)

        return mru

    def addMostRecentlyUsedTerm(self, term):
        """Add a term to the list of MRU values.
        """
        tokens = self.getMostRecentlyUsedTokens()
        try:
            tokens.remove(term.token)
        except ValueError:
            pass

        tokens.insert(0, term.token)
        del tokens[10:] # TODO should this constant be configurable?

    def queryViewApplied(self):
        """Determine if a query view was used to set the value of the field.
        """
        for name, queryview in self.queryviews:
            if name+'.apply' in self.request:
                return True
    
    def __call__(self):
        zc.resourcelibrary.need('zc.form.mruwidget')
        result = ['<div class="value">']
        value = self._value()
        field = self.context
        term = None
        if value != field.missing_value:
            try:
                term = self.terms.getTerm(value)
            except LookupError:
                pass # let the "missing" term value from above be used
            else:
                self.addMostRecentlyUsedTerm(term)

        mru_terms = self.getMostRecentlyUsedTerms()
        queries_id = self.name + '.queries'

        # should the query views be visible?
        if (self.request.form.get(queries_id+'.visible') == 'yes'
        and not self.queryViewApplied()) or not mru_terms:
            queries_style = ''
            queries_visible = 'yes'
        else:
            queries_style = 'display: none;'
            queries_visible = 'no'

        result.append('<input type="hidden" name="%s.visible" '
                      'id="%s.visible" value="%s">' 
                      % (queries_id, queries_id, queries_visible))

        if mru_terms:
            result.append('<select name="%s" id="%s">' %
                          (self.name, self.name))
            for mru_term in mru_terms:
                if term is not None and mru_term.token == term.token:
                    selected = ' selected="selected"'
                else:
                    selected = ''
                result.append('  <option value="%s"%s>%s</option>'
                              % (cgi.escape(mru_term.token), 
                                 selected, 
                                 cgi.escape(mru_term.title)))

            result.append('</select>')

            result.append('  <input type="button" '
              'name="%s.mru_expand_button" ' % self.name +
              'onclick="javascript:zc_mruwidget_toggleQueriesDisplay(\'%s\')"'
              % queries_id + ' value="...">')

        result.append('  <input type="hidden" name="%s.displayed" value="y">'
                      % self.name)
        
        result.append('  <div class="queries" id="%s" style="%s">'
                      % (queries_id, queries_style))
        for name, queryview in self.queryviews:
            result.append('    <div class="query">')
            result.append('      <div class="queryinput">')
            result.append(queryview.render(name))
            result.append('      </div> <!-- queryinput -->')

            qresults = queryview.results(name)
            if qresults:
                result.append('      <div class="queryresults">\n%s' %
                              self._renderResults(qresults, name))
                result.append('      </div> <!-- queryresults -->')
            result.append('    </div> <!-- query -->')
        result.append('  </div> <!-- queries -->')
        result.append('</div> <!-- value -->')
        return '\n'.join(result)



##############################################################################
#
# Copyright (c) 2003-2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""tests

$Id: tests.py 621 2005-02-15 19:40:07Z jim $
"""

from unittest import TestCase, TestSuite, main, makeSuite
from zope.testing.doctestunit import DocTestSuite

def test_suite():
    return TestSuite((
        DocTestSuite("zc.form.field"),
        ))

if __name__ == '__main__':
    main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Views for exceptions used by the Schema product

$Id: exceptionviews.py 3629 2005-10-06 21:01:27Z gary $
"""

from zope import interface, component, i18n
from zope.app.form.browser.interfaces import IWidgetInputErrorView
from zope.publisher.interfaces.browser import IBrowserRequest
from cgi import escape
from zope.schema.interfaces import ValidationError
from zope.app.form.interfaces import ConversionError
from zope.exceptions.interfaces import UserError


class AbstractErrorView(object):
    interface.implements(IWidgetInputErrorView)

    def __init__(self, context, request):
        self.context, self.request = context, request

    def snippet(self):
        """Convert an invariant error to an html snippet."""
        msg = self.context.args[0]
        msg = i18n.translate(msg, context=self.request, default=msg)
        return u'<span class="error">%s</span>' % escape(unicode(msg))


class ValidationErrorView(AbstractErrorView):
    component.adapts(ValidationError, IBrowserRequest)


class ConversionErrorView(AbstractErrorView):
    component.adapts(ConversionError, IBrowserRequest)


class UserErrorView(AbstractErrorView):
    component.adapts(UserError, IBrowserRequest)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""timezone widget

$Id: tzwidget.py 3872 2005-11-05 04:41:55Z gary $
"""
import pytz

from zope import interface, component
import zope.publisher.interfaces.browser
from zope.app import pagetemplate
import zope.interface.common.idatetime

import zc.form.browser.mruwidget
import zc.form.interfaces

ALL_TIMEZONES = frozenset(pytz.all_timezones)

# XXX argh.
import pytz.tzinfo
from zope.security.checker import NamesChecker
from zope.interface.common.idatetime import ITZInfo
names = set(ITZInfo.names(all=True))
names.add('zone')
names.add('localize')
checker = NamesChecker(names)
pytz.UTC.__Security_checker__ = checker
pytz.tzinfo.BaseTzInfo.__Security_checker__ = checker
# end argh.

class TimeZoneWidget(zc.form.browser.mruwidget.MruSourceInputWidget):
        
    def getMostRecentlyUsedTerms(self):
        mru = super(TimeZoneWidget, self).getMostRecentlyUsedTerms()
        # add ones from locale
        territory = self.request.locale.id.territory
        if territory:
            try:
                choices = pytz.country_timezones(territory)
            except KeyError:
                pass
            else:
                already = set(term.token for term in mru)
                additional = sorted(t for t in choices if t not in already)
                mru.extend(zc.form.interfaces.Term(t.replace('_', ' '), t)
                             for t in additional)
        return mru

class TimeZoneQueryView(object):
    interface.implements(
            zope.app.form.browser.interfaces.ISourceQueryView)

    component.adapts(zc.form.interfaces.AvailableTimeZones,
                     zope.publisher.interfaces.browser.IBrowserRequest)

    def __init__(self, source, request):
        self.context = source
        self.request = request

    _render = pagetemplate.ViewPageTemplateFile('timezone_queryview.pt')
    def render(self, name):
        return self._render(field_name=name+'.searchstring',
                            button_name=name+'.search')

    def results(self, name):
        if not (name+'.search' in self.request):
            return None

        searchstring = self.request[name+'.searchstring'].lower()
        timezones = []
        searchstring = searchstring.strip().lower().replace(' ', '_') # regex
        for tz in ALL_TIMEZONES:
            if searchstring in tz.lower():
                timezones.append(pytz.timezone(tz))
        return timezones



##############################################################################
#
# Copyright (c) 2003-2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Alternate base classes for IBrowserWidget implementations.

The base classes provided here implement the IBrowserWidget API and
provide a simpler API that derived classes are expected to implement.
"""

from xml.sax.saxutils import escape, quoteattr

from zope.interface import implements
from zope.schema.interfaces import ValidationError

from zope.app import zapi
from zope.app.form import InputWidget
from zope.app.form.browser.widget import BrowserWidget
from zope.app.form.browser.interfaces import IBrowserWidget
from zope.app.form.interfaces import IWidget, IInputWidget, WidgetInputError

from zc.form.i18n import _

_msg_missing_single_value_display = _(
    _("widget-missing-single-value-for-display"),   "")
_msg_missing_multiple_value_display = _(
    _("widget-missing-multiple-value-for-display"), "")

_msg_missing_single_value_edit = _(
    _("widget-missing-single-value-for-edit"),   "(no value)")
_msg_missing_multiple_value_edit = _(
    _("widget-missing-multiple-value-for-edit"), "(no value)")


class BaseWidget(BrowserWidget, InputWidget):
    # Note to previous users of widgetapi:
    # .translate -> ._translate; .__prefix -> ._prefix;  NullValue ->
    # ._data_marker; .__initial_value and .__calculated_value -> replaced
    # with ._data (because zope.app.form.utility.setUpWidget behavior changed
    # for the better)
    implements(IBrowserWidget, IInputWidget)

    _initialized = False
    _error = None
    _display = False # set this to True if you are using this for a display
                     # widget

    # Form management methods.
    # Subclasses should not need to override these.

    def getInputValue(self):
        if not self._initialized:
            self._initialize()

        if self._error is not None:
            raise self._error

        value = self._data
        field = self.context

        # allow missing values only for non-required fields
        if value == field.missing_value and not field.required:
            return value

        # value must be valid per the field contraints
        try:
            field.validate(value)
        except ValidationError, v:
            self._error = WidgetInputError(
                self.context.__name__, self.context.title, v)
            raise self._error
        return value

    def hasInput(self):
        if self._display:
            return False
        marker_name = self.name + "-marker"
        return marker_name in self.request.form

    def _initialize(self):
        self._initialized = True
        self.initialize()
        if not self._renderedValueSet():
            if self.hasInput():
                self._data = self.loadValueFromRequest()
            else: # self._data is self._data_marker but no input in request
                self._data = self.context.default

    def applyChanges(self, content):
        field = self.context
        value = self.getInputValue()
        change = field.query(content, self) != value
        if change:
            field.set(content, value)
            # Dynamic fields may change during a set, so re-get their value;
            # this is a larger Zope3 problem which is solved here for now.
            self._data = field.get(content)
        return change

    # Rendering methods:
    # (These should not need to be overridden.)

    def __call__(self):
        if not self._initialized:
            self._initialize()
        marker = self._get_marker()
        return marker + self.render(self._data)
    
    def hidden(self):
        if not self._initialized:
            self._initialize()
        marker = self._get_marker()
        return marker + self.renderHidden(self._data)

    def _get_marker(self):
        marker_name = self.name + "-marker"
        return "<input type='hidden' name='%s' value='x' />\n" % marker_name

    # API for subclasses to implement:

    def initialize(self):
        """Initialize internal data structures needed by the widget.

        This method should not load values from the request.

        Derived classes should call the base class initialize() before
        performing specialized initialization.  This requirement is
        waived for classes which inherit directly from, and *only*
        from, BaseWidget.
        """

    def loadValueFromRequest(self):
        """Load the value from data in the request."""
        raise NotImplementedError(
            "BaseWidget subclasses must implement loadValueFromRequest()")

    def render(self, value):
        raise NotImplementedError(
            "BaseWidget subclasses must implement render()")

    def renderHidden(self, value):
        "render a hidden widget"


class BaseVocabularyWidget(BaseWidget):

    query = None
    queryview = None

    def __init__(self, field, vocabulary, request):
        """Initialize the widget."""
        # only allow this to happen for a bound field
        assert field.context is not None
        self.vocabulary = vocabulary
        super(BaseVocabularyWidget, self).__init__(field, request)

    # Helpers used by the vocabulary widget machinery;
    # these should not be overriden.

    def setQuery(self, query, queryview):
        assert self.query is None
        assert self.queryview is None
        assert query is not None
        assert queryview is not None
        self.query = query
        self.queryview = queryview

        # Use of a hyphen to form the name for the query widget
        # ensures that it won't clash with anything else, since
        # field names are normally Python identifiers.
        queryview.setName(self.name + "-query")

    def initialize(self):
        """Make sure the query view has a chance to initialize itself."""
        if self.queryview is not None:
            self.queryview.initialize()

    def loadValueFromRequest(self):
        """Load the value from data in the request.

        If self.queryview is not None, this method is responsible for
        calling the query view's performAction() method with the value
        loaded, and returning the result::

            value = ...load value from request data...
            if self.queryview is not None:
                value = self.queryview.performAction(value)
            return value
        """
        return super(BaseVocabularyWidget, self).loadValueFromRequest()

    # Convenience method:

    def convertTokensToValues(self, tokens):
        """Convert a list of tokens to a list of values.

        If an invalid token is encountered, WidgetInputError is raised.
        """
        L = []
        for token in tokens:
            try:
                term = self.vocabulary.getTermByToken(token)
            except LookupError:
                raise WidgetInputError(
                    self.context.__name__,
                    self.context.title,
                    "token %r not found in vocabulary" % token)
            else:
                L.append(term.value)
        return L


class BaseVocabularyDisplay(BaseVocabularyWidget):
    
    _display = True

    def render(self, value):
        if value in (NullValue, None):
            # missing single value
            return self.translate(_msg_missing_single_value_display)
        else:
            return self.renderTerm(self.vocabulary.getTerm(value))

    def renderTerm(self, term):
        """Return textual presentation for term."""
        raise NotImplementedError("BaseVocabularyMultiDisplay subclasses"
                                  " must implement renderTerm()")

    def _get_marker(self):
        return ""


class BaseVocabularyMultiDisplay(BaseVocabularyDisplay):
    """Base class for display widgets of multi-valued vocabulary fields."""

    def render(self, value):
        if not value:
            # missing multiple value
            return self.translate(_msg_missing_multiple_value_display)
        else:
            pattern = ("<li>%s\n"
                       "    <input type='hidden' name=%s value=%s /></li>")
            vocabulary = self.vocabulary
            L = []
            name = quoteattr(self.name)
            for v in value:
                term = vocabulary.getTerm(v)
                L.append(pattern % (self.renderTerm(term), name,
                                    quoteattr(term.token)))
            return ("<%s class=%s id=%s>\n%s\n</%s>"
                    % (self.containerElementType,
                       quoteattr(self.containerCssClass),
                       quoteattr(self.name),
                       "\n".join(L),
                       self.containerElementType))

    containerCssClass = "values"


class BaseVocabularyBagDisplay(BaseVocabularyMultiDisplay):
    """Base class for display widgets of unordered multi-valued
    vocabulary fields."""

    containerElementType = "ul"


class BaseVocabularyListDisplay(BaseVocabularyMultiDisplay):
    """Base class for display widgets of ordered multi-valued
    vocabulary fields."""

    containerElementType = "ol"


class BaseQueryView(object):

    name = None
    widget = None
    _initialized = False

    def __init__(self, context, request):
        self.context = context
        self.request = request

    # Methods called by the vocabulary widget construction machinery;
    # subclasses should not need to override these.

    def setName(self, name):
        assert not self._initialized
        assert not name.endswith(".")
        assert self.name is None
        self.name = name

    def setWidget(self, widget):
        assert not self._initialized
        assert self.widget is None
        assert widget is not None
        self.widget = widget

    # Methods which may be overriden by subclasses:

    def initialize(self):
        """Initialization which does not require reading the request.

        Derived classes should call the base class initialize() before
        performing specialized initialization.
        """
        # Should loading from the request happen here?
        assert self.name is not None
        assert self.widget is not None
        assert not self._initialized
        self._initialized = True

    def renderResults(self, value):
        """Render query results if we have any, otherwise return an
        empty string.
        """
        results = self.getResults()
        if results is None:
            return ""
        else:
            return self.renderQueryResults(results, value)

    # Methods which should be overriden by subclasses:

    def performAction(self, value):
        """Perform any modifications to the value based on user actions.

        This method should be overriden if the query view provides any
        actions which can modify the value of the field.
        """
        return value

    # Methods which must be overriden by subclasses:

    def getResults(self):
        """Perform the query, or return None.

        The return value should be None if there is no query to
        execute, or an object that can be rendered as a set of results
        by renderQueryResults().

        If the query results in an empty set of results, some value
        other than None should be used to represent the results so
        that renderQueryResults() can provide a helpful message.
        """
        raise NotImplementedError(
            "BaseQueryView subclasses must implement getResults()")

    def renderInput(self):
        """Render the input area of the query view."""
        raise NotImplementedError(
            "BaseQueryView subclasses must implement renderInput()")

    def renderQueryResults(self, results, value):
        """Render the results returned by getResults()."""
        raise NotImplementedError(
            "BaseQueryView subclasses must implement renderQueryResults()")



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""tests of custom widgets defined in Schema

$Id: tests.py 3629 2005-10-06 21:01:27Z gary $
"""
import unittest
import re

from zope.testing import doctest, module
from zope.app.testing import placelesssetup
import zope.traversing.adapters

from zope.interface import implements
from zope.schema.interfaces import ValidationError
from zope.schema import TextLine, Int
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.app.testing import ztapi
from zope.app.form.browser.textwidgets import TextWidget, IntWidget
from zope.schema.interfaces import ITextLine, IInt
from zope.app.form.interfaces import IInputWidget
from zope.publisher.browser import TestRequest

from zope.configuration import xmlconfig
import zope.app.security
import zc.form.browser
from zc.form.field import Union
import zc.form.field 
from zc.form.browser.unionwidget import UnionWidget
from zope.testing.doctestunit import pprint

from zope import component
import zope.app.form.interfaces
import zope.app.form.browser
import zope.schema.interfaces
import zope.publisher.interfaces.browser

class TestUnionWidget(placelesssetup.PlacelessSetup, unittest.TestCase):

    def setUp(self):
        super(TestUnionWidget, self).setUp()
        # XXX cheating: should not rely on these. :-/
        ztapi.provideView(
            ITextLine, IBrowserRequest, IInputWidget, '', TextWidget)
        ztapi.provideView(
            IInt, IBrowserRequest, IInputWidget, '', IntWidget)
        from zc.form.browser.unionwidget import default_template
        component.provideAdapter(default_template, name='default')
        component.provideAdapter(
            zope.traversing.adapters.DefaultTraversable,
            [None],
            )

    def test_render(self):
        request = TestRequest()
        field = Union(
            (TextLine(title=u"Name", min_length=5),
             Int(title=u"Age", min=0)),
            title=u"Simple Identifier",
            __name__='identifier')
        widget = UnionWidget(field, request)
        widget.setPrefix('field')
        output = widget()
        self.failUnless('<table' in output)
        self.failUnless('Age' in output)
        self.failUnless('Name' in output)
        self.failUnless(re.search(
            '''type=['"]radio['"]''', output))
        self.failUnless(re.search(
            '''name=['"]field.identifier['"]''', output))
        self.failUnless(re.search(
            '''id=["']field.identifier-00['"]''', output))
        self.failUnless(re.search(
            '''id=["']field.identifier-01['"]''', output))
        self.failUnless(re.search(
            '''name=["']field.identifier.unioned_00['"]''', output))
        self.failUnless(re.search(
            '''name=["']field.identifier.unioned_01['"]''', output))
        self.failIf(re.search(
            '''id=["']field.identifier-02['"]''', output))
        self.failIf(re.search(
            '''checked\s*=\s*['"]checked['"]''', output))
        field = Union(
            (TextLine(title=u"Name", min_length=5),
             Int(title=u"Age", min=0)),
            title=u"Simple Identifier",
            __name__='identifier',
            required=False)
        widget = UnionWidget(field, request)
        widget.setPrefix('field')
        output = widget()
        self.failUnless(re.search(
            '''id=["']field.identifier-02['"]''', output))
        self.failUnless(re.search(
            '''checked\s*=\s*['"]checked['"]''', output))

    def test_use_default_for_not_selected(self):
        # test use_default_for_not_selected = True
        request = TestRequest()
        # the default selection shoud be the the option field which has the
        # value of None
        field = Union(
            (zc.form.field.TextLine(
                    title=u"New Password", missing_value=u'',
                    default_getter=lambda x: u'secret password'),
             zc.form.field.Option(
                    title=u"No Change", value_getter=lambda x: None)),
            title=u"Change Password",
            missing_value=u'',
            use_default_for_not_selected=True,
            __name__='identifier')
        widget = UnionWidget(field, request)
        widget.setPrefix('field')
        output = widget()
        # remove double whitespaces
        normalized_output = " ".join(output.split())
        
        # the value of the textline field should be the default_getter's
        # result
        value_attr_of_textline = re.search(
            '<input.*id="field.identifier.unioned_00".*(value=".*").*></div>',
            normalized_output).groups()[0]
        self.failUnless('secret password' in value_attr_of_textline)
        
        # the radio button of the option field should be selected
        radio_option_field = re.search(
            '<input.*id="field.identifier-01"(.*)/> </td>',
            normalized_output).groups()[0]
        self.failUnless('checked="checked"' in radio_option_field)

    def test_evaluate(self):
        request = TestRequest()
        request.form.update({
            'field.identifier-marker':'x',
            'field.identifier.unioned_00':u'Foo Bar',
            'field.identifier':'0'})
        field = Union(
            (TextLine(title=u"Name", min_length=5),
             Int(title=u"Age", min=0)),
            title=u"Simple Identifier",
            __name__='identifier')
        widget = UnionWidget(field, request)
        widget.setPrefix('field')
        output = widget() # to initialize
        self.assertEquals(widget.loadValueFromRequest(), u'Foo Bar')

def zcml(s, execute=True):
    context = xmlconfig.file('meta.zcml',
                             package=zc.form.browser)
    return xmlconfig.string(s, context, execute=execute)

def pageSetUp(test):
    placelesssetup.setUp(test)
    component.provideAdapter(
        zope.traversing.adapters.DefaultTraversable,
        [None],
        )

def test_suite():
    suite = unittest.makeSuite(TestUnionWidget)
    suite.addTest(doctest.DocFileSuite(
        'exceptionviews.txt',
        setUp=placelesssetup.setUp, tearDown=placelesssetup.tearDown))
    suite.addTest(
        doctest.DocFileSuite(
            'combinationwidget.txt',
            setUp=pageSetUp,
            tearDown=placelesssetup.tearDown),)
    return suite

if __name__ == "__main__":
    unittest.main(defaultTest='test_suite')



from setuptools import setup, find_packages

setup(
    name="zc.form",
    version="0.1.3",
    packages=find_packages('src'),
    package_dir={'':'src'},
    namespace_packages=['zc'],
    include_package_data=True,
    install_requires=[
        'pytz',
        'setuptools',
        'zc.resourcelibrary',
        'zc.sourcefactory',
        'ZODB3',
        'zope.annotation',
        'zope.app.principalannotation',
        'zope.app.catalog',
        'zope.app.form',
        'zope.app.pagetemplate',
        'zope.app.zapi',
        'zope.cachedescriptors',
        'zope.component',
        'zope.exceptions',
        'zope.formlib',
        'zope.i18n',
        'zope.i18nmessageid',
        'zope.index',
        'zope.interface',
        'zope.publisher',
        'zope.schema',
        # extras
        'zope.app.security',
        'zope.app.appsetup',
        'zope.app.securitypolicy',
        'zope.app.testing',
        'zope.configuration',
        'zope.testing',
        'zope.traversing',
        'zope.app.component',
        'zope.app.zcmlfiles', # sigh
        ],
    zip_safe=False
    )



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""functional test harness for zc.mruwidget
"""
import os

from zope import interface, component
from zope.app.testing.functional import FunctionalDocFileSuite, ZCMLLayer
from zope.testing import doctest
import zope.schema.interfaces

colors = ['red', 'green', 'cerulean blue']

class AvailableColors(object):
    interface.implements(zope.schema.interfaces.ISource)

    def __contains__(self, value):
        return value in colors


class Term(object):
    interface.implements(zope.schema.interfaces.ITitledTokenizedTerm)

    def __init__(self, title, token):
        self.title = title
        self.token = token


class ColorTerms(object):
    """Term and value support needed by query widgets."""

    interface.implements(zope.app.form.browser.interfaces.ITerms)
    component.adapts(zope.schema.interfaces.ISource,
                     zope.publisher.interfaces.browser.IBrowserRequest)

    def __init__(self, source, request):
        self.request = request

    def getTerm(self, value):
        token = value + '_token'
        title = value.capitalize()
        return Term(title, token)

    def getValue(self, token):
        return token.split('_')[0]

class SimpleColorSourceQueryView(object):
    interface.implements(zope.app.form.browser.interfaces.ISourceQueryView)
    component.adapts(zope.schema.interfaces.ISource,
                     zope.publisher.interfaces.browser.IBrowserRequest)

    def __init__(self, source, request):
        self.context = source
        self.request = request

    def render(self, name):
        return '<query view for colors>'

    def results(self, name):
        if not (name+'.search' in self.request):
            return None
        searchstring = self.request[name+'.searchstring']
        return [c for c in colors if searchstring in c]


here = os.path.dirname(os.path.realpath(__file__))

ZCFormLayer = ZCMLLayer(
    os.path.join(here, "ftesting.zcml"), __name__, "ZCFormLayer")

def setUp(test):
    component.provideAdapter(ColorTerms)
    component.provideAdapter(SimpleColorSourceQueryView)
    

def test_suite():
    suite = FunctionalDocFileSuite("mruwidget.txt", 
            globs={'AvailableColors': AvailableColors()},
            optionflags=doctest.NORMALIZE_WHITESPACE+doctest.ELLIPSIS,
            setUp=setUp,
            )
    suite.layer = ZCFormLayer
    return suite



##############################################################################
#
# Copyright (c) 2003-2005 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""interfaces.py: interfaces for the package

$Id: interfaces.py 4634 2006-01-06 20:21:15Z fred $"""
import datetime
import pytz

from zope import interface, component, schema
import zope.publisher.interfaces.browser
import zope.schema.interfaces
import zope.app.form.browser.interfaces
from zc.form.i18n import _

class IExtendedField(zope.schema.interfaces.IField):

    constraints = schema.Tuple(
        title=_("Constraints"),
        description=_("""A tuple of callables taking the combination field
        and the value, that should raise an exception if the values do not
        match the constraint.  Can assume that value is not the missing_value,
        and has a len equal to the number of sub-fields."""))

    def default_getter(context):
        "returns the default value"

    default = interface.Attribute(
        """if default_getter has been set, returns the result of that call;
        otherwise returns whatever value has been set as the default.""")

class IOptionField(IExtendedField):
    """Field with excatly one predefined value

    Caution: The value will not get displayed by the widget of this field.
    """

    value = interface.Attribute(
        """the value for this field; one and only one of value and
        value_getter must be non-None""")
    
    value_getter = interface.Attribute(
        """a callable, taking a context, return the option's value; or None.
        one and only one of value and value_getter must be non-None""")
    
    identity_comparison = schema.Bool(
        description=_("""Whether validation comparison should be identity
        (as opposed to equality) based"""))
    
    def getValue():
        "return value for option field"

class IUnionField(IExtendedField):
    u"""A field that may have one of many field types of values.  
    Order is important, in that the first field from left to right that
    validates a value is considered to be the "active" field."""
    
    fields = schema.Tuple(
        title=_("Composite Fields"),
        description=_("""\
            The possible schema fields that may describe the data"""))

    use_default_for_not_selected = schema.Bool(
        description=_(
            """When displaying the Union field in the browser the fields
            which are not selected will have no value (i. e. the field's
            missing_value.

            With this attribute set the default value of the field will
            be displayed instead.

            Default: False"""))
    
    def validField(value):
        u"returns first valid field for the given value, or None"

class ICombinationField(IExtendedField):
    u"""A field that describes a combination of two or more fields"""

    fields = schema.Tuple(
        title=_("Composite Fields"),
        description=_("""\
            The schema fields that may describe the data"""))

class IExtendedTextLineField(IExtendedField, zope.schema.interfaces.ITextLine):
    "TextLine field extended with IExtendedField capabilities"

class AvailableTimeZones(object):
    interface.implements(zope.schema.interfaces.ISource)

    def __contains__(self, value):
        return isinstance(value, datetime.tzinfo)

class Term:
    interface.implements(zope.schema.interfaces.ITitledTokenizedTerm)

    def __init__(self, title, token):
        self.title = title
        self.token = token

class TimeZoneTerms:
    """Term and value support needed by query widgets."""

    interface.implements(zope.app.form.browser.interfaces.ITerms)
    component.adapts(AvailableTimeZones,
                     zope.publisher.interfaces.browser.IBrowserRequest)

    def __init__(self, source, request):
        self.request = request

    def getTerm(self, value):
        token = value.zone
        title = token.replace('_', ' ')
        return Term(title, token)

    def getValue(self, token):
        return pytz.timezone(token)

class IHTMLSnippet(zope.schema.interfaces.IText):
    """HTML excerpt that can be placed within an HTML document's body element.
    
    Snippet should have no dangling open tags.
    
    XHTML preferred; field may have version attribute in future.
    """

class IHTMLDocument(zope.schema.interfaces.IText):
    """HTML Document.
    
    XHTML preferred; field may have version attribute in future.
    """
